@@ -36,6 +36,32 @@
 #include <QLoggingCategory>
 #include <QDir>
 #include <QLockFile>
+#include <QDirIterator>
+#include <QDesktopServices>
+
+#ifdef Q_OS_UNIX
+#include <QDBusError>
+#include <QDBusReply>
+#include <QDBusInterface>
+#include <QDBusPendingCall>
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
+#include <QProcess>
+#endif
+#include <QDir>
+#include <QLockFile>
+#include <QDirIterator>
+#include <QDesktopServices>
+
+#ifdef Q_OS_UNIX
+#include <QDBusError>
+#include <QDBusReply>
+#include <QDBusInterface>
+#include <QDBusPendingCall>
+#include <QDBusConnection>
+#include <QDBusConnectionInterface>
+#include <QProcess>
+#endif
 
 #include <private/qguiapplication_p.h>
 #include <qpa/qplatformservices.h>
@@ -46,6 +72,39 @@
 #include <unistd.h>
 #endif
 
+#ifdef Q_OS_UNIX
+class EnvReplaceGuard
+{
+public:
+    EnvReplaceGuard(const int uid);
+    ~EnvReplaceGuard();
+
+    char *m_backupLogName;
+    char *m_backupHome;
+    bool initialized = false;
+};
+
+EnvReplaceGuard::EnvReplaceGuard(const int uid)
+{
+    if (struct passwd *pwd = getpwuid(static_cast<__uid_t>(uid))) {
+        m_backupLogName = getenv("LOGNAME");
+        m_backupHome = getenv("HOME");
+
+        setenv("LOGNAME", pwd->pw_name, 1);
+        setenv("HOME", pwd->pw_dir, 1);
+        initialized = true;
+    }
+}
+
+EnvReplaceGuard::~EnvReplaceGuard()
+{
+    if (initialized) {
+        setenv("LOGNAME", m_backupLogName, 1);
+        setenv("HOME", m_backupHome, 1);
+    }
+}
+#endif
+
 DGUI_BEGIN_NAMESPACE
 
 #ifdef QT_DEBUG
@@ -105,6 +164,13 @@ DGuiApplicationHelperPrivate::DGuiApplicationHelperPrivate(DGuiApplicationHelper
 
 void DGuiApplicationHelperPrivate::init()
 {
+    D_Q(DGuiApplicationHelper);
+
+    systemTheme = new DPlatformTheme(0, q);
+    // 直接对应到系统级别的主题, 不再对外提供为某个单独程序设置主题的接口.
+    // 程序设置自身主题相关的东西皆可通过 setPaletteType 和 setApplicationPalette 实现.
+    appTheme = systemTheme;
+
     if (qGuiApp) {
         initApplication(qGuiApp);
     } else {
@@ -118,15 +184,6 @@ void DGuiApplicationHelperPrivate::initApplication(QGuiApplication *app)
 {
     D_Q(DGuiApplicationHelper);
 
-    if (!systemTheme) {
-        // 需要在QGuiApplication创建后再创建DPlatformTheme，否则DPlatformTheme无效.
-        // qGuiApp->platformFunction()会报警告，并返回nullptr.
-        systemTheme = new DPlatformTheme(0, q);
-        // 直接对应到系统级别的主题, 不再对外提供为某个单独程序设置主题的接口.
-        // 程序设置自身主题相关的东西皆可通过 setPaletteType 和 setApplicationPalette 实现.
-        appTheme = systemTheme;
-    }
-
     // 跟随application销毁
     qAddPostRoutine(staticCleanApplication);
 
@@ -161,8 +218,11 @@ void DGuiApplicationHelperPrivate::staticInitApplication()
     if (!_globalHelper.exists())
         return;
 
-    if (DGuiApplicationHelper *helper = _globalHelper->m_helper.load())
-        helper->d_func()->initApplication(qGuiApp);
+    if (DGuiApplicationHelper *helper = _globalHelper->m_helper.load()) {
+        // systemTheme未创建时说明DGuiApplicationHelper还未初始化
+        if (helper->d_func()->systemTheme)
+            helper->d_func()->initApplication(qGuiApp);
+    }
 }
 
 void DGuiApplicationHelperPrivate::staticCleanApplication()
@@ -387,6 +447,45 @@ QColor DGuiApplicationHelper::adjustColor(const QColor &base,
     return new_color;
 }
 
+/*!
+  \brief 调整图片整体像素颜色.
+
+  \note 取值范围均为 -100 ~ 100 ,当三原色参数为-100时，颜色为黑色，参数为100时，颜色为白色.
+  以透明度( alphaFloat )为例,当参数为负数时基础色的 alphaFloat 值减少，现象偏向透明, 参数为正数alphaFloat 值增加，现象偏不透明
+  \a base 基础色
+  \a hueFloat 色调
+  \a saturationFloat 饱和度
+  \a lightnessFloat 亮度
+  \a redFloat 红色
+  \a greenFloat 绿色
+  \a blueFloat 蓝色
+  \a alphaFloat Alpha通道(透明度)
+  \return 经过调整的图片
+ */
+QImage DGuiApplicationHelper::adjustColor(const QImage &base, qint8 hueFloat, qint8 saturationFloat, qint8 lightnessFloat, qint8 redFloat, qint8 greenFloat, qint8 blueFloat, qint8 alphaFloat)
+{
+    if (base.isNull())
+        return base;
+
+    if (!hueFloat && !saturationFloat && !lightnessFloat && !redFloat
+            && !greenFloat && !blueFloat && !alphaFloat)
+        return base;
+
+    QImage dest = base;
+    for (int y = 0; y < dest.height(); ++y) {
+        const QRgb *rgb = reinterpret_cast<const QRgb *>(base.scanLine(y));
+        for (int x = 0; x < dest.width(); ++x) {
+            QColor base = QColor::fromRgba(rgb[x]);
+            if (base.alpha() == 0)
+                continue;
+            QColor color = adjustColor(base, hueFloat, saturationFloat, lightnessFloat,
+                                       redFloat, greenFloat, blueFloat, alphaFloat);
+            dest.setPixel(x, y, color.rgba());
+        }
+    }
+    return dest;
+}
+
 /*!
   \brief 将两种颜色混合，合成新的颜色.
 
@@ -1191,6 +1290,7 @@ bool DGuiApplicationHelper::setSingleInstance(const QString &key, DGuiApplicatio
     }
 
     if (new_server) {
+        qCDebug(dgAppHelper) << "===> new server <===" << _d_singleServer->serverName() << getpid();
         QObject::connect(_d_singleServer, &QLocalServer::newConnection, qApp, [] {
             QLocalSocket *instance = _d_singleServer->nextPendingConnection();
             // 先发送数据告诉新的实例自己收到了它的请求
@@ -1247,6 +1347,51 @@ void DGuiApplicationHelper::setSingelInstanceInterval(int interval)
     DGuiApplicationHelperPrivate::waitTime = interval;
 }
 
+/*!
+ * \brief Determine whether it's a user manual for this application.
+ * \return
+ */
+bool DGuiApplicationHelper::hasUserManual() const
+{
+#ifdef Q_OS_LINUX
+    auto loadManualFromLocalFile = [] () -> bool {
+        const QString appName = qApp->applicationName();
+        if (!QFile::exists("/usr/bin/dman"))
+            return false;
+
+        bool dmanDataExists = false;
+        // search all subdirectories
+        QString strManualPath = "/usr/share/deepin-manual";
+        QDirIterator it(strManualPath, QDirIterator::Subdirectories);
+        while (it.hasNext()) {
+            QFileInfo file(it.next());
+            if (file.isDir() && file.fileName().contains(appName, Qt::CaseInsensitive)) {
+                dmanDataExists = true;
+                break;
+            }
+
+            if (file.isDir())
+                continue;
+        }
+        return dmanDataExists;
+    };
+
+    QDBusConnection conn = QDBusConnection::sessionBus();
+    if (conn.interface()->isServiceRegistered("com.deepin.Manual.Search")) {
+        QDBusInterface manualSearch("com.deepin.Manual.Search",
+                                    "/com/deepin/Manual/Search",
+                                    "com.deepin.Manual.Search");
+        if (manualSearch.isValid()) {
+            QDBusReply<bool> reply = manualSearch.call("ManualExists", qApp->applicationName());
+            return reply.value();
+        }
+    }
+    return loadManualFromLocalFile();
+#else
+    return false;
+#endif
+}
+
 void DGuiApplicationHelper::setAttribute(DGuiApplicationHelper::Attribute attribute, bool enable)
 {
     if (attribute < Attribute::ReadOnlyLimit) {
@@ -1324,6 +1469,52 @@ void DGuiApplicationHelper::setPaletteType(DGuiApplicationHelper::ColorType pale
     Q_EMIT paletteTypeChanged(paletteType);
 }
 
+/*!
+ * \brief Open manual for this application.
+ */
+void DGuiApplicationHelper::handleHelpAction()
+{
+    if (!hasUserManual()) {
+        return;
+    }
+#ifdef Q_OS_LINUX
+    QString appid = qApp->applicationName();
+
+    // new interface use applicationName as id
+    QDBusInterface manual("com.deepin.Manual.Open",
+                          "/com/deepin/Manual/Open",
+                          "com.deepin.Manual.Open");
+    QDBusReply<void> reply = manual.call("ShowManual", appid);
+    if (reply.isValid())  {
+        return;
+    }
+    // fallback to old interface
+    QProcess::startDetached("dman", QStringList() << appid);
+#else
+    qWarning() << "not support dman now";
+#endif
+}
+
+void DGuiApplicationHelper::openUrl(const QString &url)
+{
+#ifdef Q_OS_UNIX
+    // workaround for pkexec apps
+    bool ok = false;
+    const int pkexecUid = qEnvironmentVariableIntValue("PKEXEC_UID", &ok);
+
+    if (ok)
+    {
+        EnvReplaceGuard _env_guard(pkexecUid);
+        Q_UNUSED(_env_guard);
+
+        QDesktopServices::openUrl(url);
+    }
+    else
+#endif
+    {
+        QDesktopServices::openUrl(url);
+    }
+}
 DGUI_END_NAMESPACE
 
 #include "moc_dguiapplicationhelper.cpp"
