@@ -57,6 +57,13 @@ DEFINE_CONST_CHAR(windowBlurPaths);
 DEFINE_CONST_CHAR(windowWallpaperParas);
 DEFINE_CONST_CHAR(autoInputMaskByClipPath);
 
+DEFINE_CONST_CHAR(resolve_mask);
+enum PropRole {
+    WindowRadius,
+
+    // TO BE CONTINUE
+};
+
 // functions
 DEFINE_CONST_CHAR(setWmBlurWindowBackgroundArea);
 DEFINE_CONST_CHAR(setWmBlurWindowBackgroundPathList);
@@ -71,6 +78,18 @@ DEFINE_CONST_CHAR(setEnableNoTitlebar);
 DEFINE_CONST_CHAR(isEnableNoTitlebar);
 DEFINE_CONST_CHAR(clientLeader);
 
+static void resolve(QObject *obj, PropRole role)
+{
+    int mask = obj->property(_resolve_mask).toInt();
+    obj->setProperty(_resolve_mask, (mask |= 1 << role));
+}
+
+static bool resolved(QObject *obj, PropRole role)
+{
+    int mask = obj->property(_resolve_mask).toInt();
+    return mask & (1 << role);
+}
+
 static void setWindowProperty(QWindow *window, const char *name, const QVariant &value)
 {
     if (!window)
@@ -590,7 +609,8 @@ static void initWindowRadius(QWindow *window)
 
     setWindowProperty(window, _windowRadius, radius);
     window->connect(theme, &DPlatformTheme::windowRadiusChanged, window, [=] (int radius) {
-        setWindowProperty(window, _windowRadius, radius);
+        if (!resolved(window, PropRole::WindowRadius))
+            setWindowProperty(window, _windowRadius, radius);
     }, Qt::UniqueConnection);
 }
 
@@ -632,6 +652,9 @@ bool DPlatformHandle::setEnabledNoTitlebarForWindow(QWindow *window, bool enable
     if (!(isDXcbPlatform() || isDWaylandPlatform()))
         return false;
 
+    if (isEnabledNoTitlebar(window) == enable)
+        return true;
+
     QFunctionPointer enable_no_titlear = nullptr;
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 4, 0)
@@ -640,7 +663,7 @@ bool DPlatformHandle::setEnabledNoTitlebarForWindow(QWindow *window, bool enable
 
     if (enable_no_titlear) {
         bool ok = (*reinterpret_cast<bool(*)(QWindow*, bool)>(enable_no_titlear))(window, enable);
-        if (ok) {
+        if (ok && enable) {
             if (window->handle()) {
                 initWindowRadius(window);
             } else {
@@ -1136,6 +1159,7 @@ WId DPlatformHandle::windowLeader()
 void DPlatformHandle::setWindowRadius(int windowRadius)
 {
     setWindowProperty(m_window, _windowRadius, windowRadius);
+    resolve(m_window, PropRole::WindowRadius);
 }
 
 void DPlatformHandle::setBorderWidth(int borderWidth)
